#!/usr/bin/perl
#
######################################################################
# Copyright 1995, 1996 Ipsilon Networks, Inc. All rights reserved.
#
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################
# Copyright 1994 Steven E. Brenner
# Unpublished work.
# Permission granted to use and modify this library so long as the
# copyright above is maintained, modifications are documented, and
# credit is given for any use of the library.
# 
# The ReadInput() and ParseData() routines are derivatives
# of Steve Brenner's ReadParse routine in cgi-lib.pl.
######################################################################
#
# This library contains common subroutines 
# used in the IP Switch Navigator.
#

# Globals

#
# Read input sent by browser to cgi program.
# This routine reads the input received via the web GET or POST method
# and returns the input in the variable: $input.
# Call with:
#	&ReadInput(*input);
#
sub ReadInput {
  local(*input) = @_;
  local($len);

  if ($ENV{'REQUEST_METHOD'} eq "GET") {
        $input = $ENV{'QUERY_STRING'};       # Using GET
  }
  else {
        $len = $ENV{'CONTENT_LENGTH'};       # Using POST
        if ($len > "34000") {                # Input size should never be absurd
                die "input buffer overflow, stopped";
        }
        read(STDIN, $input, $len);

  }
  return length($input);
}

#
# This routine takes the input string in $input, parses it into 
# name/value pairs and puts the result into the *nvarray associative array.
# Call with:
#	&ParseData(*nvarray, $input);
#
sub ParseData {
  local(*nvarray, $input) = @_;
  local($name, $value);
  local($i);

  @nvarray = split(/&/, $input);

  foreach $i (0 .. $#nvarray)
  {
        $nvarray[$i] =~ s/\+/ /g;

        # Split on the first '='
        ($name, $value) = split(/=/, $nvarray[$i],2);

        # Translate web-special chars (i.e. international) into unsigned char.
        $name =~ s/%(..)/pack("C",hex($1))/eg;
        $value =~ s/%(..)/pack("C",hex($1))/eg;

        # Store name=value pair
        if (defined($nvarray{$name})) {
                $nvarray{$name} .= "\0";
        }
        $nvarray{$name} .= $value;
#       print "Name: $name, Value: $value\n";
  }
}

sub SetUpHelp {
    $ihelp = $GLOBAL{'IH'};
    if (!ihelp || $ihelp eq "off") {
	$ihelp = "off";
	$nhelp = "on";
    }elsif ($ihelp eq "on") {
	$nhelp = "off";
    }
}

#
# This routine prints a message header with 2 carriage returns.
# Call with:
#	&PrintMsgHdr("string");
#
sub PrintMsgHdr {
  local($msg) = @_;

  print stdout "<b>$msg</b><br> <br>\n";
}

#
# This routine prints out a simple message.
#	&PrintMsg("string");
#
sub PrintMsg {
  local($msg) = @_;

  print stdout "<b>$msg</b><br>\n";
}

# 
# Print out web page header or trailer 
# Call with:
#	&PrintButtonBar("HEADER", "TitleString");
# or
#	&PrintButtonBar("TRAILER");
#
# The HEADER call will print out the button bar with the Digital logo
# in the middle.  It also displays the "TitleString" on the page with:
# 	<title>TitleString</title>
#	<h2>TitleString</h2>
#
sub PrintButtonBar0 {
  local($type, $msg) = @_;
  local($row, $i);
  local($fnprefix, $fnpostfix);
  local($numbuttons);
  #
  # button array row = filename, gif, button label
  local(@buttons) = (
		     "", "home.gif", "Home",
		     "http:/cgi-bin/switch", "lighttiny.gif", "Top",
		     "diag.pl", "diagtool.gif", "Diag",
     "http:/cgi-bin/html_page?TEMPLATE=events&","bellsmall.gif","Events",
		     "flows.pl", "flows.gif", "Flows",
     "http:/cgi-bin/html_page?TEMPLATE=rsmon&","rsmon.gif","RS Stats",
		     "voy.guide/Monitor_tasks.fm.html", "help.gif", "Help"
		    );

  if ($type eq "HEADER" || $type eq "NPHEADER" ) {
      print stdout "Content-type: text/html\n\n";

      print stdout "<html>\n";
      print stdout "<head><title>Digital: $msg</title></head>\n";
      
      print stdout "<BODY BGCOLOR=\"#F0F0F0\">\n";
      print stdout "<body>\n";
      print stdout "<center>\n";
      print stdout "<a href=\"http://www.networks.digital.com\">";
      print stdout "<img border=0 src=\"/images/voyagersmall.gif\" alt=\"clearVISN IP Switch Manager\"><br>";
      print stdout "</a>\n";
      print stdout "</center>\n";
  }
  
  print stdout "<center>\n";
  print stdout "<table>\n";
  print stdout "<tr>\n";

  #
  # Go through the buttons array, and display each button
  #
  $numbuttons = (@buttons) / 3;
  $row=0;
  $cell=0;
  for ($row=0; $row<$numbuttons; $row++) {
	if (($IPSDEVTYPE eq "gateway") && ($buttons[$cell+2] eq "Flows")) {
	        $cell = 3 * ($row + 1);
	        next;
	}
	print stdout "<td align=center>\n";

	($fnprefix, $fnpostfix) = split(/\./, $buttons[$cell]);

	# print out hyperlink
	if ($fnpostfix eq "pl") {
		print stdout "<a href=\"/cgi-bin/$buttons[$cell]\">\n";
	}
	elsif ($fnpostfix eq "html" || $fnpostfix eq "" ||
	       $buttons[$cell] eq "") {
		if ($buttons[$cell+2] =~ /Home/) {
			print stdout "<a href=\"/$buttons[$cell]\">\n";
		}
		elsif ($buttons[$cell+2] =~ /Help/) {
			print stdout "<a href=\"/$buttons[$cell]\">\n";
		}
		else {
			print stdout "<a href=\"$buttons[$cell]\">\n";
		}
	}

	# Print out image
	print stdout "<img border=0 src=/images/$buttons[$cell+1] alt=\"$buttons[$cell+2]\"></a>\n";

	print stdout "</td>\n";
	$cell = 3 * ($row + 1);
  }

  print stdout "</tr>\n";
  print stdout "</table>\n";
  if (($type eq "HEADER" || $type eq "NPHEADER")) {
      print stdout "<hr size=4>\n";
      print stdout "<font size=+2><b>$msg</b></font>\n";
  }
  print stdout "</center>\n";
  
  if ($type eq "HEADER" || $type eq "NPHEADER") {
	print stdout "<hr size=4>\n";
  }
  if ($type eq "TRAILER") {
	print stdout "</body></html>\n";
  }
}

# The HEADER call will print out the button bar with the Digital logo
# in the middle.  It also displays the "TitleString" on the page with:
# 	<title>TitleString</title>
# The TRAILER Call will have
#	link to qos main page
#	and a link to $BackPage
#
sub PrintHeadToe {
  local($type, $msg, $doc, $up, $help_page, $top) = @_;
  local($row, $i);
  local($fnprefix, $fnpostfix);
  local($numbuttons);
  &SetUpHelp;
  if ($type eq "HEADER" || $type eq "NPHEADER") {
      print "Content-type: text/html\n\n";
      print "<Html>\n";
      print "<head><title>Digital: $msg</title></head>\n";
      print "<Body bgcolor=\"f0f0f0\">\n";
      print "<center>\n";
      print "<img border=0 src=\"/images/config/voyagersmall.gif\" alt=------Digital------>\n";
      print "<br>\n";
      print "<table><tr>\n";
  }elsif ($type eq "TRAILER") {
      print "<center\n";
      print "<br>\n";
      print "<table><tr>";
  }

  print "<td><a href=\"/\"><img border=0 src=\"/images/config/home.gif\" alt=\"Home\"></a>\n";
  print "<td><a href=\"/cgi-bin/switch?IH=$ihelp\"> <img border=0 src=\"/images/lighttiny.gif\" alt=\"SwitchView\"></a>\n";
  print "<td><a href=\"/cgi-bin/diag.pl?IH=$ihelp\"><img border=0 src=\"/images/diagtool.gif\" alt=\"Diag\"></a>\n";
  print "<td><a href=\"/cgi-bin/html_page?TEMPLATE=events&IH=$ihelp\"><img border=0 src=\"/images/bellsmall.gif\" alt=\"Up\"></a>\n";
  print "<td><a href=\"/cgi-bin/flows.pl?IH=$ihelp\"><img border=0 src=\"/images/flows.gif\" alt=\"Flows\"></a>\n";
  print "<td><a href=\"/cgi-bin/html_page?TEMPLATE=rsmon&IH=$ihelp\"><img border=0 src=\"/images/config/rsmon.gif\" alt=\"Up\"></a>\n";

  if (!ihelp || $ihelp eq "off") {
      print "<td><a href=\"$help_page\"><img border=0 src=\"/images/config/ihelp-on.gif\" alt=\"IH\"></a>\n"; 
  }else {
      print "<td><a href=\"$help_page\"><img border=0 src=\"/images/config/ihelp-off.gif\" alt=\"IH\"></a>\n"; 
  }
  print "<td><a href=\"http:/voy.guide/Monitor_tasks.fm.html\"><img border=0 src=\"/images/config/help.gif\" alt=\"Help\"></a>\n"; 
  print "</tr><tr>\n";

  print "</tr></table>\n";

  if ($type eq "HEADER") {
      print "<hr>\n";
      print "<font size=+2>$msg</font>\n";
      print "<hr>\n";
      print "</center>\n";
  }elsif ($type eq "TRAILER") {
      print "</center>\n";
      print "</body></html>\n";
  }
}

sub PrintHeadToe2{
  local($type, $msg, $doc, $up, $help_page, $top) = @_;
  local($row, $i);
  local($fnprefix, $fnpostfix);
  local($numbuttons);

  if ($type eq "HEADER" || $type eq "NPHEADER") {
      print "Content-type: text/html\n\n";
      print "<Html>\n";
      print "<head><title>Digital: $msg</title></head>\n";
      print "<Body bgcolor=\"f0f0f0\">\n";
      print "<center>\n";
      print "<img border=0 src=\"/images/config/voyagersmall.gif\" alt=------Digital------>\n";
      print "<br>\n";
      print "<table><tr>\n";
  }elsif ($type eq "TRAILER") {
      print "<center\n";
      print "<br>\n";
      print "<table><tr>";
  }

  print "<td><a href=\"/\"><img border=0 src=\"/images/config/home.gif\" alt=\"Home\"></a>\n";
  print "<td><a href=\"/cgi-bin/switch?IH=$ihelp\"> <img border=0 src=\"/images/lighttiny.gif\" alt=\"SwitchView\"></a>\n";
  print "<td><a href=\"/cgi-bin/diag.pl?IH=$ihelp\"><img border=0 src=\"/images/diagtool.gif\" alt=\"Diag\"></a>\n";
  print "<td><a href=\"/cgi-bin/html_page?TEMPLATE=events&IH=$ihelp\"><img border=0 src=\"/images/bellsmall.gif\" alt=\"Up\"></a>\n";
  print "<td><a href=\"/cgi-bin/flows.pl?IH=off\"><img border=0 src=\"/images/flows.gif\" alt=\"Flows\"></a>\n";
  print "<td><a href=\"/cgi-bin/html_page?TEMPLATE=rsmon&IH=$ihelp\"><img border=0 src=\"/images/config/rsmon.gif\" alt=\"Up\"></a>\n";

  print "<td><a href=\"http:/voy.guide/Monitor_tasks.fm.html\"><img border=0 src=\"/images/config/help.gif\" alt=\"Help\"></a>\n"; 
  
  print "</tr><tr>\n";

  print "</tr></table>\n";

  if ($type eq "HEADER" || $type eq "NPHEADER") {
      print "<hr>\n";
      print "<font size=+2>$msg</font>\n";
      print "<hr>\n";
      print "</center>\n";
  }elsif ($type eq "TRAILER") {
      print "</center>\n";
      print "</body></html>\n";
  }
}

sub display_help {
    local($path) = @_;
    if ($ihelp eq "on") {
        print "<img src=\"/images/config/helpbar.gif\"><br>";
	print "<font -= 4><br>";
        open(FD, $path);
        while (<FD>) {
            print;
        }
        close(FD);
	print "<font += 4>";
        print "<img src=\"/images/config/bar.gif\"><br>";
    }
}

sub IpswitchName {
        local ($hostname);

	open (FD, "/usr/bin/uname -n |"); chop($hostname=<FD>); close(FD);
	if ($hostname)
		{ return "$hostname"; }
	return "IP Switch";
}

# 
# Test IP address against "", 0-leading name and "0.0.0.0" 
# Call with:
#	&IsIPAddr("1.2.3.4");
# or
#	&IsIPAddr("servo");
# Return:
#	1 for good IP address
#	0 for bad IP Address
#
sub IsIPAddr {
    local($ipaddr) = @_;
   
    if ( $ipaddr =~ /^[A-Za-z][-\w]*$/ ) {
	return 1;
    }
    elsif ( $ipaddr =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ ) {
              for ("$1", "$2", "$3", "$4") { 
	    	   if ($_ > 255) { 
		      return 0; 
		   }
              }   

	return 1;
    }
    elsif ( $ipaddr =~ /^(.*)\.(.*)\.(.*)/ ) {
	      for ("$1", "$2", "$3") {
		 if (/^[ ]*$/) { 
		      return 0; 
		   }
	       }

	return 1;
    }
}

#
# Get input value from @GLOBAL after calling ReadData() and ParseData()
# Call with:
#       &GetValue($GLOBOL[n]);
# Return:
#       Returned Value, that is extracted from @GLOBAL
#
sub GetValue {
    local ($globVal) = @_; 
    local (@array, $retVal); 

    @array = split(/=/,$globVal);
    $retVal = @array[1];
    return $retVal;
}

1;
